#include <string>
#include <boost/regex.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>
#include "http_endpoint.h"
#include "static_data.h"
#include "http_error.h"
#include "utils.h"

namespace simple_http
{
	http_endpoint::http_endpoint(const std::string& address, unsigned short port, bool ssl)
		:address_(address), port_(port), ssl_(ssl)
	{}

	http_endpoint http_endpoint::from_string(const std::string& full_name)
	{
		boost::smatch what;

		try
		{
			if(!boost::regex_match(full_name, what, full_name_regex))
			{
				if(!boost::regex_match(full_name, what, full_name_ip_regex) ||
					!utils::is_ip_address(what[2].str()))
					throw http_error("Invalid HTTP(S) endpoint!", http_error::incorrect_http_endpoint);
			}
		}
		catch(const boost::regex_error&)
		{
			throw http_error("Invalid HTTP(S) endpoint!", http_error::incorrect_http_endpoint);
		}

		bool ssl = boost::iequals(what[1].str(), "https");
		unsigned short port = ssl ? 443 : 80;

		if(what[3].matched)
		{
			port = boost::lexical_cast<unsigned short>(what[3].str().c_str() + 1);
			if(!port)
				throw http_error("Invalid HTTP(S) endpoint!", http_error::incorrect_http_endpoint);
		}

		http_endpoint ret(what[2].str(), port, ssl);

		if(what[4].matched)
			ret.uri(what[4].str());
		else
			ret.uri("/");

		if(what[5].matched)
			ret.parameters(what[5].str().c_str() + 1);

		return ret;
	}

	std::string http_endpoint::address() const
	{
		return address_;
	}

	unsigned short http_endpoint::port() const
	{
		return port_;
	}

	bool http_endpoint::ssl() const
	{
		return ssl_;
	}

	void http_endpoint::address(const std::string& address)
	{
		address_ = address;
	}

	void http_endpoint::port(unsigned short port)
	{
		port_ = port;
	}

	void http_endpoint::ssl(bool ssl)
	{
		ssl_ = ssl;
	}

	std::string http_endpoint::uri() const
	{
		return uri_;
	}

	std::string http_endpoint::parameters() const
	{
		return parameters_;
	}

	void http_endpoint::uri(const std::string& uri)
	{
		uri_ = uri;
	}

	void http_endpoint::parameters(const std::string& parameters)
	{
		parameters_ = parameters;
	}
}
