#include <string>
#include <map>
#include <functional>
#include "simple_http_defs.h"

namespace simple_http
{
	class simple_http_impl;
	class multipart_file;
	class cookie_manager;
	class http_endpoint;
	class http_sync_connection
	{
	public:
		http_sync_connection();
		explicit http_sync_connection(const std::string& address, unsigned short port = 80, bool ssl = false);
		explicit http_sync_connection(const http_endpoint& ep);
		~http_sync_connection();

		std::string get_address() const;
		unsigned short get_port() const;
		bool is_ssl_connection() const;
		std::string get_user_agent() const;

		void set_user_agent(const std::string& user_agent = "");
		void set_connection_params(const std::string& address, unsigned short port = 80, bool ssl = false);
		void set_connection_params(const http_endpoint& ep);

		std::string get(const std::string& uri, const std::string& parameters = "", const std::string& cookies = "", const std::string& referer = "");
		std::string post(const std::string& uri, const std::string& parameters = "", const std::string& cookies = "", const std::string& referer = "");
		std::string query(const std::string& method, const std::string& uri, const std::string& parameters = "", const std::string& cookies = "", const std::string& referer = "");
		
		std::string multipart_post(const std::string& uri, const std::string& multipart_file_parameter_name, multipart_file& file, const std::string& parameters = "", const std::string& cookies = "", const std::string& referer = "");
		std::string multipart_post(const std::string& uri, std::map<std::string, multipart_file*>& files, const std::string& parameters = "", const std::string& cookies = "", const std::string& referer = "");
		std::string multipart_post(const std::string& uri, const std::string& parameters = "", const std::string& cookies = "", const std::string& referer = "");

		
		std::string get(const http_endpoint& ep, const std::string& cookies = "", const std::string& referer = "");
		std::string post(const http_endpoint& ep, const std::string& cookies = "", const std::string& referer = "");
		std::string query(const std::string& method, const http_endpoint& ep, const std::string& cookies = "", const std::string& referer = "");
		
		std::string multipart_post(const http_endpoint& ep, const std::string& multipart_file_parameter_name, multipart_file& file, const std::string& cookies = "", const std::string& referer = "");
		std::string multipart_post(const http_endpoint& ep, std::map<std::string, multipart_file*>& files, const std::string& cookies = "", const std::string& referer = "");
		std::string multipart_post(const http_endpoint& ep, const std::string& cookies = "", const std::string& referer = "");


		bool is_connection_cache_enabled() const;
		void set_connenction_cache(bool enabled = true);

		std::string get_proxy_address() const;
		std::string get_proxy_login() const;
		std::string get_proxy_password() const;
		unsigned short get_proxy_port() const;
		proxy_type get_proxy_type() const;
		void set_proxy(const std::string& address, unsigned short port, proxy_type ptype);
		void set_proxy(const std::string& address, unsigned short port, proxy_type ptype, const std::string& login, const std::string& password);
		void clear_proxy();

		void set_query_timeout(unsigned long timeout = 60);
		unsigned long get_query_timeout() const;

		void add_query_header(const std::string& name, const std::string& value);
		bool remove_query_header(const std::string& name);
		void clear_query_headers();
		std::string get_query_header_value(const std::string& name) const;
		void get_query_headers(std::map<std::string, std::string>& list) const;

		void set_auto_cookie_manager(cookie_manager& manager);
		void remove_auto_cookie_manager();
		cookie_manager* get_auto_cookie_manager();
		const cookie_manager* get_auto_cookie_manager() const;

		void enable_auto_redirect_follow(bool follow = true);
		bool auto_redirect_follow_enabled() const;
		void enable_cross_domain_redirects(bool allow = true);
		bool cross_diomain_redirects_enabled() const;
		void set_max_redirect_count(unsigned long count = 10);
		unsigned long get_max_redirect_count() const;
		unsigned long get_recent_redirect_count() const;
		std::string get_last_redirect_domain() const;
		std::string get_last_redirect_uri() const;
		std::string get_last_redirect_line() const;
		unsigned short get_last_redirect_port() const;

		void enable_auto_urlencode(bool enable = true);
		bool auto_urlencode_enabled() const;

		template<typename Handler>
		void set_http_load_callback(Handler handler)
		{
			load_callback_ = handler;
		}

		void clear_http_load_callback();
		void clear_input_after_callback(bool clear = true);
		bool clear_input_after_callback() const;
		
		void set_callback_min_transfer_amount(unsigned long bytes);
		unsigned long get_callback_min_transfer_amount() const;

		void set_callback_option(http_callback_option option = get_everything);
		http_callback_option get_callback_option() const;
	
	private:
		simple_http_impl* impl;
		http_sync_connection(const http_sync_connection&);
		http_sync_connection& operator=(const http_sync_connection&);
		std::string internal_query(const std::string& method, const std::string& uri, const std::string& parameters, const std::string& cookies, const std::string& referer, bool multipart = false, std::map<std::string, multipart_file*>* files = 0);

		std::function<bool(const std::string&)> load_callback_;
	};
}
