#include <string>
#include <boost/asio.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/thread/mutex.hpp>
#include <functional>
#include "simple_http_impl.h"
#include "http_error.h"
#include "socket_not_encrypted.h"
#include "simple_http_socket_worker.h"
#include "socket_ssl.h"
#include "static_data.h"

namespace simple_http
{
	simple_http_impl::simple_http_impl()
		:port_(80), connect_cache_enabled_(true), ssl_(false), connect_endpoint_cached_(false),
		proxy_type_(proxy_none),
		query_timeout_(60),
		cookie_mgr_(0),
		follow_redirects_(false),
		allow_cross_domain_redirects_(true),
		max_redirect_count_(10),
		current_redirect_count_(0),
		last_redirect_port_(0),
		auto_urlencode_(false),
		clear_input_after_callback_(true),
		callback_option_(get_everything),
		min_transfer_amount_(1)
	{}

	simple_http_impl::simple_http_impl(const std::string& address, unsigned short port, bool ssl)
		:address_(address), port_(port), ssl_(ssl),
		connect_cache_enabled_(true), connect_endpoint_cached_(false),
		proxy_type_(proxy_none),
		query_timeout_(60),
		cookie_mgr_(0),
		follow_redirects_(false),
		allow_cross_domain_redirects_(true),
		max_redirect_count_(10),
		current_redirect_count_(0),
		last_redirect_port_(0),
		auto_urlencode_(false),
		clear_input_after_callback_(true),
		callback_option_(get_everything),
		min_transfer_amount_(1)
	{
#ifndef SSL_SUPPORT
		if(ssl)
			throw http_error("SSL connections are not supported. Rebuild simple_http library with SSL_SUPPORT definition", http_error::ssl_not_supported);
#endif
	}

	void simple_http_impl::throw_connection_error()
	{
		switch(proxy_type_)
		{
		case proxy_http:
			throw http_error("Can't connect to http-proxy server", http_error::cannot_connect_to_http_proxy);
		case proxy_socks5:
			throw http_error("Can't connect to socks5-proxy server", http_error::cannot_connect_to_socks5_proxy);
		}

		throw http_error("Can't connect to server", http_error::cannot_connect);
	}

	void simple_http_impl::query(const std::string& query_type, boost::asio::streambuf& out, const std::string& uri, const std::string& parameters, const std::string& cookies, const std::string& referer, bool multipart, std::map<std::string, multipart_file*>* files,
		std::function<bool(const std::string&)>& callback)
	{
		//TODO:: work on this
		/*static*/ boost::asio::io_service ios;
		boost::scoped_ptr<simple_http_socket_worker_base> worker;

#ifdef SSL_SUPPORT
		if(ssl_)
			worker.reset(new simple_http_socket_worker<socket_ssl>(ios, this, callback));
		else
#endif
			worker.reset(new simple_http_socket_worker<socket_not_encrypted>(ios, this, callback));

		worker->query(query_type, out, uri, parameters, cookies, referer, multipart, files);

		//if(ios.stopped())
		//	ios.reset();

		ios.run();
	}
}
