#pragma once
#include <string>
#include <boost/asio/ip/address.hpp>
#include <boost/asio/ip/tcp.hpp>
#include <boost/noncopyable.hpp>
#include <boost/asio/streambuf.hpp>
#include <functional>
#include "simple_http_defs.h"

namespace simple_http
{
	class cookie_manager;
	class multipart_file;
	class simple_http_impl : boost::noncopyable
	{
	public:
		simple_http_impl();
		explicit simple_http_impl(const std::string& address, unsigned short port = 80, bool ssl = false);

	public:
		std::string address_, user_agent_;
		unsigned short port_;

		bool ssl_;

		boost::asio::ip::tcp::endpoint cached_endpoint_;
		bool connect_cache_enabled_, connect_endpoint_cached_;

		std::string proxy_address_;
		unsigned short proxy_port_;
		proxy_type proxy_type_;
		std::string proxy_login_, proxy_password_;

		unsigned long query_timeout_;

		std::map<std::string, std::string> query_headers_;

		cookie_manager* cookie_mgr_;

		bool follow_redirects_, allow_cross_domain_redirects_;

		unsigned long max_redirect_count_, current_redirect_count_;

		std::string last_redirect_domain_, last_redirect_uri_, last_redirect_line_;
		unsigned short last_redirect_port_;

		bool auto_urlencode_;
		
		bool clear_input_after_callback_;
		http_callback_option callback_option_;
		unsigned long min_transfer_amount_;
		
		void query(const std::string& query_type, boost::asio::streambuf& out, const std::string& uri, const std::string& parameters, const std::string& cookies, const std::string& referer, bool multipart, std::map<std::string, multipart_file*>* files, std::function<bool(const std::string&)>& callback);

		void throw_connection_error();
	};
}
