#pragma once
#include <string>
#include <map>
#include <boost/asio.hpp>
#include <boost/shared_ptr.hpp>
#include <functional>

namespace simple_http
{
	class simple_http_impl;
	class multipart_file;
	class simple_http_socket_worker_base
	{
	public:
		enum socks5_auth_step
		{
			socks5_write_auth_types_request,
			socks5_read_auth_types_request,
			socks5_auth,
			socks5_read_auth_answer,
			socks5_check_auth_answer,
			socks5_connect_request,
			socks5_read_connect_answer,
			socks5_check_connect_answer
		};

		enum https_proxy_auth_step
		{
			https_proxy_send_connect,
			https_connect_read_connect_answer,
			https_proxy_check_connect_answer
		};

	public:
		simple_http_socket_worker_base(boost::asio::io_service& ios, simple_http_impl* impl, std::function<bool(const std::string&)>& callback);
		virtual ~simple_http_socket_worker_base();
		void query(const std::string& query_type, boost::asio::streambuf& out, const std::string& uri, const std::string& parameters, const std::string& cookies, const std::string& referer, bool multipart, std::map<std::string, multipart_file*>* files);

	protected:
		void clean_request_buffer();
		void cache_endpoint(boost::asio::ip::tcp::resolver::iterator endpoint_iterator);

		void handle_resolve(const boost::system::error_code& ec, boost::asio::ip::tcp::resolver::iterator endpoint_iterator);
		void handle_connect_socks5_pre_auth(const boost::system::error_code& ec, boost::asio::ip::tcp::resolver::iterator endpoint_iterator);
		void handle_connect_http_proxy_pre_auth(const boost::system::error_code& ec, boost::asio::ip::tcp::resolver::iterator endpoint_iterator);
		void handle_read(const boost::system::error_code& ec, std::size_t bytes);
		void handle_connect_socks5_auth(const boost::system::error_code& ec, socks5_auth_step step);
		void handle_connect_http_proxy_auth(const boost::system::error_code& ec, https_proxy_auth_step step);
		void handle_handshake(const boost::system::error_code& ec);
		void handle_write_files(const boost::system::error_code& ec, bool write_next);
		void check_for_timeout(const boost::system::error_code& ec);

		virtual void connect(boost::asio::ip::tcp::resolver::iterator endpoint_iterator) = 0;
		virtual void handle_write(const boost::system::error_code& ec) = 0;
		virtual void read_socks_answer(std::size_t transfer, socks5_auth_step next_step) = 0;
		virtual void write_socks_answer(socks5_auth_step next_step) = 0;
		virtual void handle_connect(const boost::system::error_code& ec, boost::asio::ip::tcp::resolver::iterator endpoint_iterator) = 0;
		virtual void write_request() = 0;
		virtual void write_file_data(bool write_next) = 0;
		virtual void timeout_handler(const boost::system::error_code& ec) = 0;
		virtual void read_http_proxy_answer(https_proxy_auth_step next_step) = 0;
		virtual void write_http_proxy_answer(https_proxy_auth_step next_step) = 0;

	protected:
		char socks5_buf[10];
		boost::asio::streambuf request_;
		boost::asio::streambuf* out_;
		simple_http_impl* impl_;
		std::function<bool(const std::string&)>& load_callback_;
		unsigned long read_ptr_;
		bool first_read_;

	private:
		std::string boundary_;

		boost::shared_ptr<boost::asio::ip::tcp::resolver> resolver_;

		const std::string* query_type_, *uri_, *parameters_, *cookies_, *referer_;
		bool multipart_;
		std::map<std::string, multipart_file*>* files_;
		std::map<std::string, multipart_file*>::iterator file_iterator_;
		boost::asio::deadline_timer timer_;

	private:
		static const unsigned long file_buffer_size = 1024;
	};
}
